import 'dart:convert';
import 'dart:io';
import 'package:http/http.dart' as http;
import 'package:moru_payment_gateway/dto/request/access_validate.dart';
import 'package:moru_payment_gateway/dto/request/user_validate.dart';
import 'package:moru_payment_gateway/dto/request/pay_data.dart';
import 'package:moru_payment_gateway/dto/response/validated_access_res.dart';
import 'package:moru_payment_gateway/dto/response/validated_user_res.dart';
import 'package:moru_payment_gateway/exception/payment_exception.dart';
import '../constants.dart';

class PaymentHandler {
  late final http.Client _httpClient;
  final String baseNewsUrl = Constants.BASE_URL;

  PaymentHandler({required httpClient}) : _httpClient = httpClient;

  Future<ValidatedAccessRes> validateAccess(
      {required AccessValidate clientData}) async {
    try {
      final validateAccessApi =
          Uri.parse("${baseNewsUrl + Constants.VALIDATE_ACCESS_KEY}");

      Map<String, String> requestBody = clientData.toJson();
      final List<int> amount = [int.parse(clientData.amount)];
      var request = http.MultipartRequest('POST', validateAccessApi)
        ..fields.addAll(requestBody);
      request.files.add(http.MultipartFile.fromBytes('amount', amount));
      var response = await request.send();

      final Map<String, dynamic> accessValidatedMap =
          _checkResponseStatusCode(response);

      final ValidatedAccessRes validatedAccessRes =
          ValidatedAccessRes.fromJson(accessValidatedMap);

      return validatedAccessRes;
    } catch (_) {
      print(_.toString());
      throw SocketException('Could not process your request');
    }
  }

  Future<ValidatedUserRes> validateUser(
      {required UserValidate userValidate}) async {
    try {
      final Uri userValidatedUrl =
          Uri.parse(baseNewsUrl + "${Constants.VALIDATE_USER}");
      Map<String, String> requestBody = userValidate.toJson();
      final List<int> mobileNo = [int.parse(userValidate.mobileNo)];
      final List<int> pin = [int.parse(userValidate.pin)];
      var request = http.MultipartRequest('POST', userValidatedUrl)
        ..fields.addAll(requestBody);
      request.files.add(http.MultipartFile.fromBytes('mobile', mobileNo));
      request.files.add(http.MultipartFile.fromBytes('pin', pin));
      var response = await request.send();
      final Map<String, dynamic> userValidatedMap =
          _checkResponseStatusCode(response);

      final ValidatedUserRes validatedUserRes =
          ValidatedUserRes.fromJson(userValidatedMap);

      return validatedUserRes;
    } catch (_) {
      throw SocketException('Could not process your request');
    }
  }

  Future<bool> initiatePay({required PayData payData}) async {
    try {
      final Uri payUrl = Uri.parse(baseNewsUrl + "${Constants.VALIDATE_OTP}");

      Map<String, String> requestBody = payData.toJson();
      final List<int> otp = [int.parse(payData.otp)];
      var request = http.MultipartRequest('POST', payUrl)
        ..fields.addAll(requestBody);
      request.files.add(http.MultipartFile.fromBytes('otp', otp));
      var response = await request.send();

      final Map<String, dynamic>? payResponseMap =
          _checkResponseStatusCode(response);

      if (payResponseMap != null) return true;
      return false;
    } catch (_) {
      throw SocketException('Could not process your request');
    }
  }

  Map<String, dynamic> _checkResponseStatusCode(
      http.StreamedResponse response) {
    final Map<String, dynamic> gatewayResMap = jsonDecode(response.toString());
    switch (response.statusCode) {
      //ok code
      case 200:
        return gatewayResMap;

      case 400:
        throw BadRequest(gatewayResMap['message']);

      case 9000:
        throw InsufficientBalance(gatewayResMap['message']);

      case 9001:
        throw PinUnset(gatewayResMap['message']);

      case 9002:
        throw InvalidUser(gatewayResMap['message']);

      case 9004:
        throw InvalidOtp(gatewayResMap['message']);

      case 9007:
        throw PaymentAlreadyCompleted(gatewayResMap['message']);

      case 9008:
        throw OTPExpired(gatewayResMap['message']);

      case 401:
        throw UnauthorizedException('Unable to fetch from Api');

      case 403:
        throw UnauthorizedException(
            'Sorry you are unauthorized to access this api');

      default:
        throw UnauthorizedException(
            'Sorry you are unauthorized to access this api');
    }
  }
}
